@@ -20,6 +20,7 @@
 #ifndef TESSERACT_CCUTIL_GENERICVECTOR_H_
 #define TESSERACT_CCUTIL_GENERICVECTOR_H_
 
+#include <assert.h>
 #include <stdio.h>
 #include <stdlib.h>
 
@@ -27,6 +28,8 @@
 #include "errcode.h"
 #include "helpers.h"
 #include "ndminx.h"
+#include "serialis.h"
+#include "strngs.h"
 
 // Use PointerVector<T> below in preference to GenericVector<T*>, as that
 // provides automatic deletion of pointers, [De]Serialize that works, and
@@ -34,8 +37,13 @@
 template <typename T>
 class GenericVector {
  public:
-  GenericVector() { this->init(kDefaultVectorSize); }
-  explicit GenericVector(int size) { this->init(size); }
+  GenericVector() : size_used_(0), size_reserved_(0), data_(NULL),
+                    clear_cb_(NULL), compare_cb_(NULL) {}
+
+  GenericVector(int size, T init_val) {
+    init(size);
+    init_to_size(size, init_val);
+  }
 
   // Copy
   GenericVector(const GenericVector& other) {
@@ -45,7 +53,7 @@ class GenericVector {
   GenericVector<T> &operator+=(const GenericVector& other);
   GenericVector<T> &operator=(const GenericVector& other);
 
-  virtual ~GenericVector();
+  ~GenericVector();
 
   // Reserve some memory.
   void reserve(int size);
@@ -54,11 +62,19 @@ class GenericVector {
 
   // Resizes to size and sets all values to t.
   void init_to_size(int size, T t);
+  // Resizes to size without any initialization.
+  void resize_no_init(int size) {
+    reserve(size);
+    size_used_ = size;
+  }
 
   // Return the size used.
   int size() const {
     return size_used_;
   }
+  int size_reserved() const {
+    return size_reserved_;
+  }
 
   int length() const {
     return size_used_;
@@ -71,7 +87,10 @@ class GenericVector {
 
   // Return the object from an index.
   T &get(int index) const;
+  T &back() const;
   T &operator[](int index) const;
+  // Returns the last object and removes it.
+  T pop_back();
 
   // Return the index of the T object.
   // This method NEEDS a compare_callback to be passed to
@@ -88,6 +107,10 @@ class GenericVector {
   int push_back(T object);
   void operator+=(T t);
 
+  // Push an element in the end of the array if the same
+  // element is not already contained in the array.
+  int push_back_new(T object);
+
   // Push an element in the front of the array
   // Note: This function is O(n)
   int push_front(T object);
@@ -100,11 +123,11 @@ class GenericVector {
 
   // Removes an element at the given index and
   // shifts the remaining elements to the left.
-  virtual void remove(int index);
+  void remove(int index);
 
   // Truncates the array to the given size by removing the end.
   // If the current size is less, the array is not expanded.
-  virtual void truncate(int size) {
+  void truncate(int size) {
     if (size < size_used_)
       size_used_ = size;
   }
@@ -121,13 +144,13 @@ class GenericVector {
   // All the owned callbacks are also deleted.
   // If you don't want the callbacks to be deleted, before calling clear, set
   // the callback to NULL.
-  virtual void clear();
+  void clear();
 
   // Delete objects pointed to by data_[i]
   void delete_data_pointers();
 
   // This method clears the current object, then, does a shallow copy of
-  // its argument, and finally invalidate its argument.
+  // its argument, and finally invalidates its argument.
   // Callbacks are moved to the current object;
   void move(GenericVector<T>* from);
 
@@ -142,22 +165,31 @@ class GenericVector {
   bool read(FILE* f, TessResultCallback3<bool, FILE*, T*, bool>* cb, bool swap);
   // Writes a vector of simple types to the given file. Assumes that bitwise
   // read/write of T will work. Returns false in case of error.
-  virtual bool Serialize(FILE* fp) const;
+  // TODO(rays) Change all callers to use TFile and remove deprecated methods.
+  bool Serialize(FILE* fp) const;
+  bool Serialize(tesseract::TFile* fp) const;
   // Reads a vector of simple types from the given file. Assumes that bitwise
   // read/write will work with ReverseN according to sizeof(T).
   // Returns false in case of error.
   // If swap is true, assumes a big/little-endian swap is needed.
-  virtual bool DeSerialize(bool swap, FILE* fp);
+  bool DeSerialize(bool swap, FILE* fp);
+  bool DeSerialize(bool swap, tesseract::TFile* fp);
+  // Skips the deserialization of the vector.
+  static bool SkipDeSerialize(bool swap, tesseract::TFile* fp);
   // Writes a vector of classes to the given file. Assumes the existence of
   // bool T::Serialize(FILE* fp) const that returns false in case of error.
   // Returns false in case of error.
   bool SerializeClasses(FILE* fp) const;
+  bool SerializeClasses(tesseract::TFile* fp) const;
   // Reads a vector of classes from the given file. Assumes the existence of
   // bool T::Deserialize(bool swap, FILE* fp) that returns false in case of
   // error. Also needs T::T() and T::T(constT&), as init_to_size is used in
   // this function. Returns false in case of error.
   // If swap is true, assumes a big/little-endian swap is needed.
   bool DeSerializeClasses(bool swap, FILE* fp);
+  bool DeSerializeClasses(bool swap, tesseract::TFile* fp);
+  // Calls SkipDeSerialize on the elements of the vector.
+  static bool SkipDeSerializeClasses(bool swap, tesseract::TFile* fp);
 
   // Allocates a new array of double the current_size, copies over the
   // information from data to the new location, deletes data and returns
@@ -171,6 +203,12 @@ class GenericVector {
     return data_new;
   }
 
+  // Reverses the elements of the vector.
+  void reverse() {
+    for (int i = 0; i < size_used_ / 2; ++i)
+      Swap(&data_[i], &data_[size_used_ - 1 - i]);
+  }
+
   // Sorts the members of this vector using the less than comparator (cmp_lt),
   // which compares the values. Useful for GenericVectors to primitive types.
   // Will not work so great for pointers (unless you just want to sort some
@@ -197,18 +235,20 @@ class GenericVector {
   }
   // Searches the array (assuming sorted in ascending order, using sort()) for
   // an element equal to target and returns the index of the best candidate.
-  // The return value is the largest index i such that data_[i] <= target or 0.
+  // The return value is conceptually the largest index i such that
+  // data_[i] <= target or 0 if target < the whole vector.
+  // NOTE that this function uses operator> so really the return value is
+  // the largest index i such that data_[i] > target is false.
   int binary_search(const T& target) const {
     int bottom = 0;
     int top = size_used_;
-    do {
+    while (top - bottom > 1) {
       int middle = (bottom + top) / 2;
       if (data_[middle] > target)
         top = middle;
       else
         bottom = middle;
     }
-    while (top - bottom > 1);
     return bottom;
   }
 
@@ -254,13 +294,47 @@ class GenericVector {
     return result;
   }
 
+  // Returns the index of what would be the target_index_th item in the array
+  // if the members were sorted, without actually sorting. Members are
+  // shuffled around, but it takes O(n) time.
+  // NOTE: uses operator< and operator== on the members.
+  int choose_nth_item(int target_index) {
+    // Make sure target_index is legal.
+    if (target_index < 0)
+      target_index = 0;                   // ensure legal
+    else if (target_index >= size_used_)
+      target_index = size_used_ - 1;
+    unsigned int seed = 1;
+    return choose_nth_item(target_index, 0, size_used_, &seed);
+  }
+
+  // Swaps the elements with the given indices.
+  void swap(int index1, int index2) {
+    if (index1 != index2) {
+      T tmp = data_[index1];
+      data_[index1] = data_[index2];
+      data_[index2] = tmp;
+    }
+  }
+  // Returns true if all elements of *this are within the given range.
+  // Only uses operator<
+  bool WithinBounds(const T& rangemin, const T& rangemax) const {
+    for (int i = 0; i < size_used_; ++i) {
+      if (data_[i] < rangemin || rangemax < data_[i])
+        return false;
+    }
+    return true;
+  }
+
  protected:
+  // Internal recursive version of choose_nth_item.
+  int choose_nth_item(int target_index, int start, int end, unsigned int* seed);
 
   // Init the object, allocating size memory.
   void init(int size);
 
   // We are assuming that the object generally placed in thie
-  // vector are small enough that for efficiency it makes sence
+  // vector are small enough that for efficiency it makes sense
   // to start with a larger initial size.
   static const int kDefaultVectorSize = 4;
   inT32   size_used_;
@@ -273,6 +347,45 @@ class GenericVector {
 
 namespace tesseract {
 
+// Function to read a GenericVector<char> from a whole file.
+// Returns false on failure.
+typedef bool (*FileReader)(const STRING& filename, GenericVector<char>* data);
+// Function to write a GenericVector<char> to a whole file.
+// Returns false on failure.
+typedef bool (*FileWriter)(const GenericVector<char>& data,
+                           const STRING& filename);
+// The default FileReader loads the whole file into the vector of char,
+// returning false on error.
+inline bool LoadDataFromFile(const STRING& filename,
+                             GenericVector<char>* data) {
+  bool result = false;
+  FILE* fp = fopen(filename.string(), "rb");
+  if (fp != NULL) {
+    fseek(fp, 0, SEEK_END);
+    size_t size = ftell(fp);
+    fseek(fp, 0, SEEK_SET);
+    if (size > 0) {
+      // reserve an extra byte in case caller wants to append a '\0' character
+      data->reserve(size + 1);
+      data->resize_no_init(size);
+      result = fread(&(*data)[0], 1, size, fp) == size;
+    }
+    fclose(fp);
+  }
+  return result;
+}
+// The default FileWriter writes the vector of char to the filename file,
+// returning false on error.
+inline bool SaveDataToFile(const GenericVector<char>& data,
+                           const STRING& filename) {
+  FILE* fp = fopen(filename.string(), "wb");
+  if (fp == NULL) return false;
+  bool result =
+      static_cast<int>(fwrite(&data[0], 1, data.size(), fp)) == data.size();
+  fclose(fp);
+  return result;
+}
+
 template <typename T>
 bool cmp_eq(T const & t1, T const & t2) {
   return t1 == t2;
@@ -320,58 +433,85 @@ class PointerVector : public GenericVector<T*> {
  public:
   PointerVector() : GenericVector<T*>() { }
   explicit PointerVector(int size) : GenericVector<T*>(size) { }
-  virtual ~PointerVector() {
+  ~PointerVector() {
     // Clear must be called here, even though it is called again by the base,
     // as the base will call the wrong clear.
     clear();
   }
   // Copy must be deep, as the pointers will be automatically deleted on
   // destruction.
-  PointerVector(const PointerVector& other) {
-    init(other.size());
+  PointerVector(const PointerVector& other) : GenericVector<T*>(other) {
+    this->init(other.size());
     this->operator+=(other);
   }
   PointerVector<T>& operator+=(const PointerVector& other) {
-    reserve(this->size_used_ + other.size_used_);
+    this->reserve(this->size_used_ + other.size_used_);
     for (int i = 0; i < other.size(); ++i) {
-      push_back(new T(*other.data_[i]));
+      this->push_back(new T(*other.data_[i]));
     }
     return *this;
   }
 
   PointerVector<T>& operator=(const PointerVector& other) {
-    this->truncate(0);
-    this->operator+=(other);
+    if (&other != this) {
+      this->truncate(0);
+      this->operator+=(other);
+    }
     return *this;
   }
 
   // Removes an element at the given index and
   // shifts the remaining elements to the left.
-  virtual void remove(int index) {
+  void remove(int index) {
     delete GenericVector<T*>::data_[index];
     GenericVector<T*>::remove(index);
   }
 
   // Truncates the array to the given size by removing the end.
   // If the current size is less, the array is not expanded.
-  virtual void truncate(int size) {
+  void truncate(int size) {
     for (int i = size; i < GenericVector<T*>::size_used_; ++i)
       delete GenericVector<T*>::data_[i];
     GenericVector<T*>::truncate(size);
   }
 
+  // Compact the vector by deleting elements for which delete_cb returns
+  // true. delete_cb is a permanent callback and will be deleted.
+  void compact(TessResultCallback1<bool, const T*>* delete_cb) {
+    int new_size = 0;
+    int old_index = 0;
+    // Until the callback returns true, the elements stay the same.
+    while (old_index < GenericVector<T*>::size_used_ &&
+           !delete_cb->Run(GenericVector<T*>::data_[old_index++]))
+      ++new_size;
+    // Now just copy anything else that gets false from delete_cb.
+    for (; old_index < GenericVector<T*>::size_used_; ++old_index) {
+      if (!delete_cb->Run(GenericVector<T*>::data_[old_index])) {
+        GenericVector<T*>::data_[new_size++] =
+            GenericVector<T*>::data_[old_index];
+      } else {
+        delete GenericVector<T*>::data_[old_index];
+      }
+    }
+    GenericVector<T*>::size_used_ = new_size;
+    delete delete_cb;
+  }
+
   // Clear the array, calling the clear callback function if any.
   // All the owned callbacks are also deleted.
   // If you don't want the callbacks to be deleted, before calling clear, set
   // the callback to NULL.
-  virtual void clear() {
+  void clear() {
     GenericVector<T*>::delete_data_pointers();
     GenericVector<T*>::clear();
   }
 
-  // Writes a vector of simple types to the given file. Assumes that bitwise
-  // read/write of T will work. Returns false in case of error.
-  virtual bool Serialize(FILE* fp) const {
+  // Writes a vector of (pointers to) classes to the given file. Assumes the
+  // existence of bool T::Serialize(FILE*) const that returns false in case of
+  // error. There is no Serialize for simple types, as you would have a
+  // normal GenericVector of those.
+  // Returns false in case of error.
+  bool Serialize(FILE* fp) const {
     inT32 used = GenericVector<T*>::size_used_;
     if (fwrite(&used, sizeof(used), 1, fp) != 1) return false;
     for (int i = 0; i < used; ++i) {
@@ -381,34 +521,97 @@ class PointerVector : public GenericVector<T*> {
     }
     return true;
   }
-  // Reads a vector of simple types from the given file. Assumes that bitwise
-  // read/write will work with ReverseN according to sizeof(T).
+  bool Serialize(TFile* fp) const {
+    inT32 used = GenericVector<T*>::size_used_;
+    if (fp->FWrite(&used, sizeof(used), 1) != 1) return false;
+    for (int i = 0; i < used; ++i) {
+      inT8 non_null = GenericVector<T*>::data_[i] != NULL;
+      if (fp->FWrite(&non_null, sizeof(non_null), 1) != 1) return false;
+      if (non_null && !GenericVector<T*>::data_[i]->Serialize(fp)) return false;
+    }
+    return true;
+  }
+  // Reads a vector of (pointers to) classes to the given file. Assumes the
+  // existence of bool T::DeSerialize(bool, Tfile*) const that returns false in
+  // case of error. There is no Serialize for simple types, as you would have a
+  // normal GenericVector of those.
+  // If swap is true, assumes a big/little-endian swap is needed.
   // Also needs T::T(), as new T is used in this function.
   // Returns false in case of error.
-  // If swap is true, assumes a big/little-endian swap is needed.
-  virtual bool DeSerialize(bool swap, FILE* fp) {
+  bool DeSerialize(bool swap, FILE* fp) {
     inT32 reserved;
     if (fread(&reserved, sizeof(reserved), 1, fp) != 1) return false;
     if (swap) Reverse32(&reserved);
     GenericVector<T*>::reserve(reserved);
+    truncate(0);
     for (int i = 0; i < reserved; ++i) {
       inT8 non_null;
       if (fread(&non_null, sizeof(non_null), 1, fp) != 1) return false;
       T* item = NULL;
       if (non_null) {
         item = new T;
-        if (!item->DeSerialize(swap, fp)) return false;
+        if (!item->DeSerialize(swap, fp)) {
+          delete item;
+          return false;
+        }
+        this->push_back(item);
+      } else {
+        // Null elements should keep their place in the vector.
+        this->push_back(NULL);
       }
-      push_back(item);
+    }
+    return true;
+  }
+  bool DeSerialize(bool swap, TFile* fp) {
+    inT32 reserved;
+    if (!DeSerializeSize(swap, fp, &reserved)) return false;
+    GenericVector<T*>::reserve(reserved);
+    truncate(0);
+    for (int i = 0; i < reserved; ++i) {
+      if (!DeSerializeElement(swap, fp)) return false;
+    }
+    return true;
+  }
+  // Enables deserialization of a selection of elements. Note that in order to
+  // retain the integrity of the stream, the caller must call some combination
+  // of DeSerializeElement and DeSerializeSkip of the exact number returned in
+  // *size, assuming a true return.
+  static bool DeSerializeSize(bool swap, TFile* fp, inT32* size) {
+    if (fp->FRead(size, sizeof(*size), 1) != 1) return false;
+    if (swap) Reverse32(size);
+    return true;
+  }
+  // Reads and appends to the vector the next element of the serialization.
+  bool DeSerializeElement(bool swap, TFile* fp) {
+    inT8 non_null;
+    if (fp->FRead(&non_null, sizeof(non_null), 1) != 1) return false;
+    T* item = NULL;
+    if (non_null) {
+      item = new T;
+      if (!item->DeSerialize(swap, fp)) {
+        delete item;
+        return false;
+      }
+      this->push_back(item);
+    } else {
+      // Null elements should keep their place in the vector.
+      this->push_back(NULL);
+    }
+    return true;
+  }
+  // Skips the next element of the serialization.
+  static bool DeSerializeSkip(bool swap, TFile* fp) {
+    inT8 non_null;
+    if (fp->FRead(&non_null, sizeof(non_null), 1) != 1) return false;
+    if (non_null) {
+      if (!T::SkipDeSerialize(swap, fp)) return false;
     }
     return true;
   }
 
   // Sorts the items pointed to by the members of this vector using
   // t::operator<().
-  void sort() {
-    sort(&sort_ptr_cmp<T>);
-  }
+  void sort() { this->GenericVector<T*>::sort(&sort_ptr_cmp<T>); }
 };
 
 }  // namespace tesseract
@@ -448,10 +651,11 @@ template <typename T>
 void GenericVector<T>::reserve(int size) {
   if (size_reserved_ >= size || size <= 0)
     return;
+  if (size < kDefaultVectorSize) size = kDefaultVectorSize;
   T* new_array = new T[size];
   for (int i = 0; i < size_used_; ++i)
     new_array[i] = data_[i];
-  if (data_ != NULL) delete[] data_;
+  delete[] data_;
   data_ = new_array;
   size_reserved_ = size;
 }
@@ -485,7 +689,20 @@ T &GenericVector<T>::get(int index) const {
 
 template <typename T>
 T &GenericVector<T>::operator[](int index) const {
- return data_[index];
+  assert(index >= 0 && index < size_used_);
+  return data_[index];
+}
+
+template <typename T>
+T &GenericVector<T>::back() const {
+  ASSERT_HOST(size_used_ > 0);
+  return data_[size_used_ - 1];
+}
+// Returns the last object and removes it.
+template <typename T>
+T GenericVector<T>::pop_back() {
+  ASSERT_HOST(size_used_ > 0);
+  return data_[--size_used_];
 }
 
 // Return the object from an index.
@@ -500,7 +717,7 @@ void GenericVector<T>::set(T t, int index) {
 // at the specified index.
 template <typename T>
 void GenericVector<T>::insert(T t, int index) {
-  ASSERT_HOST(index >= 0 && index < size_used_);
+  ASSERT_HOST(index >= 0 && index <= size_used_);
   if (size_reserved_ == size_used_)
     double_the_size();
   for (int i = size_used_; i > index; --i) {
@@ -555,6 +772,14 @@ int GenericVector<T>::push_back(T object) {
   return index;
 }
 
+template <typename T>
+int GenericVector<T>::push_back_new(T object) {
+  int index = get_index(object);
+  if (index >= 0)
+    return index;
+  return push_back(object);
+}
+
 // Add an element in the array (front)
 template <typename T>
 int GenericVector<T>::push_front(T object) {
@@ -583,8 +808,10 @@ GenericVector<T> &GenericVector<T>::operator+=(const GenericVector& other) {
 
 template <typename T>
 GenericVector<T> &GenericVector<T>::operator=(const GenericVector& other) {
-  this->truncate(0);
-  this->operator+=(other);
+  if (&other != this) {
+    this->truncate(0);
+    this->operator+=(other);
+  }
   return *this;
 }
 
@@ -598,7 +825,8 @@ void GenericVector<T>::set_clear_callback(TessCallback1<T>* cb) {
 // Add a callback to be called to delete the elements when the array took
 // their ownership.
 template <typename T>
-void GenericVector<T>::set_compare_callback(TessResultCallback2<bool, T const &, T const &>* cb) {
+void GenericVector<T>::set_compare_callback(
+    TessResultCallback2<bool, T const &, T const &>* cb) {
   compare_cb_ = cb;
 }
 
@@ -688,6 +916,12 @@ bool GenericVector<T>::Serialize(FILE* fp) const {
   if (fwrite(data_, sizeof(*data_), size_used_, fp) != size_used_) return false;
   return true;
 }
+template <typename T>
+bool GenericVector<T>::Serialize(tesseract::TFile* fp) const {
+  if (fp->FWrite(&size_used_, sizeof(size_used_), 1) != 1) return false;
+  if (fp->FWrite(data_, sizeof(*data_), size_used_) != size_used_) return false;
+  return true;
+}
 
 // Reads a vector of simple types from the given file. Assumes that bitwise
 // read/write will work with ReverseN according to sizeof(T).
@@ -707,6 +941,27 @@ bool GenericVector<T>::DeSerialize(bool swap, FILE* fp) {
   }
   return true;
 }
+template <typename T>
+bool GenericVector<T>::DeSerialize(bool swap, tesseract::TFile* fp) {
+  inT32 reserved;
+  if (fp->FRead(&reserved, sizeof(reserved), 1) != 1) return false;
+  if (swap) Reverse32(&reserved);
+  reserve(reserved);
+  size_used_ = reserved;
+  if (fp->FRead(data_, sizeof(T), size_used_) != size_used_) return false;
+  if (swap) {
+    for (int i = 0; i < size_used_; ++i)
+      ReverseN(&data_[i], sizeof(data_[i]));
+  }
+  return true;
+}
+template <typename T>
+bool GenericVector<T>::SkipDeSerialize(bool swap, tesseract::TFile* fp) {
+  inT32 reserved;
+  if (fp->FRead(&reserved, sizeof(reserved), 1) != 1) return false;
+  if (swap) Reverse32(&reserved);
+  return fp->FRead(NULL, sizeof(T), reserved) == reserved;
+}
 
 // Writes a vector of classes to the given file. Assumes the existence of
 // bool T::Serialize(FILE* fp) const that returns false in case of error.
@@ -719,10 +974,18 @@ bool GenericVector<T>::SerializeClasses(FILE* fp) const {
   }
   return true;
 }
+template <typename T>
+bool GenericVector<T>::SerializeClasses(tesseract::TFile* fp) const {
+  if (fp->FWrite(&size_used_, sizeof(size_used_), 1) != 1) return false;
+  for (int i = 0; i < size_used_; ++i) {
+    if (!data_[i].Serialize(fp)) return false;
+  }
+  return true;
+}
 
 // Reads a vector of classes from the given file. Assumes the existence of
 // bool T::Deserialize(bool swap, FILE* fp) that returns false in case of
-// error. Alse needs T::T() and T::T(constT&), as init_to_size is used in
+// error. Also needs T::T() and T::T(constT&), as init_to_size is used in
 // this function. Returns false in case of error.
 // If swap is true, assumes a big/little-endian swap is needed.
 template <typename T>
@@ -737,9 +1000,31 @@ bool GenericVector<T>::DeSerializeClasses(bool swap, FILE* fp) {
   }
   return true;
 }
+template <typename T>
+bool GenericVector<T>::DeSerializeClasses(bool swap, tesseract::TFile* fp) {
+  uinT32 reserved;
+  if (fp->FRead(&reserved, sizeof(reserved), 1) != 1) return false;
+  if (swap) Reverse32(&reserved);
+  T empty;
+  init_to_size(reserved, empty);
+  for (int i = 0; i < reserved; ++i) {
+    if (!data_[i].DeSerialize(swap, fp)) return false;
+  }
+  return true;
+}
+template <typename T>
+bool GenericVector<T>::SkipDeSerializeClasses(bool swap, tesseract::TFile* fp) {
+  uinT32 reserved;
+  if (fp->FRead(&reserved, sizeof(reserved), 1) != 1) return false;
+  if (swap) Reverse32(&reserved);
+  for (int i = 0; i < reserved; ++i) {
+    if (!T::SkipDeSerialize(swap, fp)) return false;
+  }
+  return true;
+}
 
 // This method clear the current object, then, does a shallow copy of
-// its argument, and finally invalindate its argument.
+// its argument, and finally invalidates its argument.
 template <typename T>
 void GenericVector<T>::move(GenericVector<T>* from) {
   this->clear();
@@ -760,4 +1045,65 @@ void GenericVector<T>::sort() {
   sort(&tesseract::sort_cmp<T>);
 }
 
+// Internal recursive version of choose_nth_item.
+// The algorithm used comes from "Algorithms" by Sedgewick:
+// http://books.google.com/books/about/Algorithms.html?id=idUdqdDXqnAC
+// The principle is to choose a random pivot, and move everything less than
+// the pivot to its left, and everything greater than the pivot to the end
+// of the array, then recurse on the part that contains the desired index, or
+// just return the answer if it is in the equal section in the middle.
+// The random pivot guarantees average linear time for the same reason that
+// n times vector::push_back takes linear time on average.
+// target_index, start and and end are all indices into the full array.
+// Seed is a seed for rand_r for thread safety purposes. Its value is
+// unimportant as the random numbers do not affect the result except
+// between equal answers.
+template <typename T>
+int GenericVector<T>::choose_nth_item(int target_index, int start, int end,
+                                      unsigned int* seed) {
+  // Number of elements to process.
+  int num_elements = end - start;
+  // Trivial cases.
+  if (num_elements <= 1)
+    return start;
+  if (num_elements == 2) {
+    if (data_[start] < data_[start + 1]) {
+      return target_index > start ? start + 1 : start;
+    } else {
+      return target_index > start ? start : start + 1;
+    }
+  }
+  // Place the pivot at start.
+  #ifndef rand_r  // _MSC_VER, ANDROID
+  srand(*seed);
+  #define rand_r(seed) rand()
+  #endif  // _MSC_VER
+  int pivot = rand_r(seed) % num_elements + start;
+  swap(pivot, start);
+  // The invariant condition here is that items [start, next_lesser) are less
+  // than the pivot (which is at index next_lesser) and items
+  // [prev_greater, end) are greater than the pivot, with items
+  // [next_lesser, prev_greater) being equal to the pivot.
+  int next_lesser = start;
+  int prev_greater = end;
+  for (int next_sample = start + 1; next_sample < prev_greater;) {
+    if (data_[next_sample] < data_[next_lesser]) {
+      swap(next_lesser++, next_sample++);
+    } else if (data_[next_sample] == data_[next_lesser]) {
+      ++next_sample;
+    } else {
+      swap(--prev_greater, next_sample);
+    }
+  }
+  // Now the invariant is set up, we recurse on just the section that contains
+  // the desired index.
+  if (target_index < next_lesser)
+    return choose_nth_item(target_index, start, next_lesser, seed);
+  else if (target_index < prev_greater)
+    return next_lesser;          // In equal bracket.
+  else
+    return choose_nth_item(target_index, prev_greater, end, seed);
+}
+
+
 #endif  // TESSERACT_CCUTIL_GENERICVECTOR_H_
